#
# ClassyTk Widgets 
# ----------------- Peter De Rijk
#
# Classy::ListBox
# ----------------------------------------------------------------------
#doc ListBox title {
#ListBox
#} index {
# Tk improvements
#} shortdescr {
# listbox with auto scroll bars, and other extras
#} descr {
# subclass of <a href="Widget.html">Widget</a><br>
# creates a listbox with automatic scrollbars, and some handy extra options.
#}
#doc {ListBox command} h2 {
#	ListBox specific methods
#}

if {[string equal $tcl_platform(platform) windows]} {
	option add *Classy::ListBox.Relief sunken widgetDefault
	option add *Classy::ListBox.BorderWidth 1 widgetDefault
	option add *Classy::ListBox.list.Relief flat widgetDefault
	option add *Classy::ListBox.list.BorderWidth 0 widgetDefault
}

bind Classy::ListBox <<MExecute>> {[Classy::mainw %W] command}
bind Classy::ListBox <<Invoke>> {[Classy::mainw %W] command}
bind Classy::ListBox <<Action>> {[Classy::mainw %W] activate @%x,%y}
bind Classy::ListBox <<ListboxSelect>> {[Classy::mainw %W] _browse}
bind Classy::ListBox <<Up>> {[Classy::mainw %W] _updown -1; break}
bind Classy::ListBox <<Down>> {[Classy::mainw %W] _updown 1; break}

# ------------------------------------------------------------------
#  Widget creation
# ------------------------------------------------------------------
Widget subclass Classy::ListBox

Classy::ListBox method init {args} {
	# REM Create object
	# -----------------
	super init
	listbox $object.list -xscrollcommand [list $object.xscroll set] -yscrollcommand [list $object.yscroll set] \
		-highlightthickness 0 \
		
	Classy::AutoScrollbar $object.xscroll -command [list $object.list xview] -orient horizontal
	Classy::AutoScrollbar $object.yscroll -command [list $object.list yview] -orient vertical
	$object.xscroll set 0.0 1.0
	$object.yscroll set 0.0 1.0
	bindtags $object [lreplace [bindtags $object] 2 0 Listbox]
	Classy::rebind $object.list $object
	grid $object.list -column 0 -row 0 -sticky nwse
	grid $object.yscroll -column 1 -row 0 -sticky nwse
	grid $object.xscroll -column 0 -row 1 -sticky nwse
	grid columnconfigure $object 0 -weight 1
	grid rowconfigure $object 0 -weight 1
	# REM Configure initial arguments
	# -------------------------------
	if {"$args" != ""} {eval $object configure $args}
	return $object
}

Classy::ListBox method destroy {} {
	private $object options
	if {![string_equal [get options(-selvariable) ""] ""]} {
		trace vdelete ::$options(-selvariable) w [list $object _traceselvar]
	}
	if {![string_equal [get options(-valuesvariable) ""] ""]} {
		trace vdelete ::$options(-valuesvariable) w [list $object _tracevaluesvar]
	}
}

# ------------------------------------------------------------------
#  Widget destroy
# ------------------------------------------------------------------

# ------------------------------------------------------------------
#  Widget options
# ------------------------------------------------------------------
Classy::ListBox chainoptions {$object.list}
Classy::ListBox chainoption -background {$object} -background {$object.list} -background
Classy::ListBox chainoption -highlightbackground {$object} -highlightbackground {$object.list} -highlightbackground
Classy::ListBox chainoption -highlightcolor {$object} -highlightcolor {$object.list} -highlightcolor

#doc {ListBox options -content} option {-content content Content} descr {
#}
Classy::ListBox addoption -content {content Content {}} {
	$object.list delete 0 end
	foreach el $value {
		$object.list insert end $el
	}
}

#doc {ListBox options -values} option {-values values Values} descr {
#}
Classy::ListBox addoption -values {values Values {}} {
}

#doc {ListBox options -command} option {-command command Command} descr {
#}
Classy::ListBox addoption -command {command Command {}} {
}

#doc {ListBox options -browsecommand} option {-browsecommand browseCommand BrowseCommand} descr {
#}
Classy::ListBox addoption -browsecommand {browseCommand BrowseCommand {}} {
}

#doc {ListBox options -endcommand} option {-endcommand endCommand EndCommand} descr {
#}
Classy::ListBox addoption -endcommand {endCommand EndCommand {}} {
}

#doc {ListBox options -selvariable} option {-selvariable selVariable SelVariable} descr {
#}
Classy::ListBox addoption -selvariable {selVariable SelVariable {}} {
	private $object options
	if {![string_equal [get options(-selvariable) ""] ""]} {
		trace vdelete ::$options(-selvariable) w [list $object _traceselvar]
	}
	if {![string_equal $value ""]} {
		catch {eval {$object set} [get ::$value]}
		trace variable ::$value w [list $object _traceselvar]
	}
}

#doc {ListBox options -valuesvariable} option {-valuesvariable valuesVariable ValuesVariable} descr {
#}
Classy::ListBox addoption -valuesvariable {valuesVariable ValuesVariable {}} {
	private $object options
	if {![string_equal [get options(-valuesvariable) ""] ""]} {
		trace vdelete ::$options(-valuesvariable) w [list $object _tracevaluesvar]
	}
	if {![string_equal $value ""]} {
		trace variable ::$value w [list $object _tracevaluesvar]
	}
}

# ------------------------------------------------------------------
#  Methods
# ------------------------------------------------------------------

Classy::ListBox chainallmethods {$object.list} listbox

#doc {ListBox command get} cmd {
#pathname get ?first? ?last?
#} descr {
#}
Classy::ListBox method get {args} {
	private $object options
	set len [llength $args]
	if {![llength $options(-values)]} {
		if {$len == 0} {
			set list ""
			foreach pos [$object.list curselection] {
				lappend list [$object.list get $pos]
			}
			return $list
		} elseif {$len == 1} {
			return [$object.list get [lindex $args 0]]
		} elseif {$len == 2} {
			return [$object.list get [lindex $args 0] [lindex $args 1]]
		} else {
			return -code error "wrong # args: should be \"$object get ?first? ?last?\""
		}
	} else {
		if {$len == 0} {
			return [list_sub $options(-values) [$object.list curselection]]
		} elseif {$len == 1} {
			return [lindex $options(-values) [$object.list index [lindex $args 0]]]
		} elseif {$len == 2} {
			return [lrange $options(-values) [$object.list index [lindex $args 0]] [$object.list index [lindex $args 1]]]
		} else {
			return -code error "wrong # args: should be \"$object get ?first? ?last?\""
		}
	}
}

#doc {ListBox command set} cmd {
#pathname set ?el? ...
#} descr {
#}
Classy::ListBox method set {args} {
	private $object options
	#set c [$object cget -content]
	set c [$object get 0 end]
	set poslist {}
	foreach el $args {
		set pos [lsearch -exact $c $el]
		if {$pos == -1} {error "element \"$el\" not in listbox"}
		lappend poslist $pos
	}
	$object selection clear 0 end
	foreach pos $poslist {
		$object.list selection set $pos
	}
	if {![string equal $options(-selvariable) ""]} {
		Classy::todo $object _setvar
	}
}

Classy::ListBox method activate {args} {
	private $object options
	focus $object.list
	set pos []
	if [catch {eval $object.list index $args} pos] {
		set c [$object cget -content]
		set pos [lsearch -exact $c [lindex $args 0]]
	}
	$object.list activate $pos
	if {![string equal $options(-selvariable) ""]} {
		Classy::todo $object _setvar
	}
	if {![string equal $options(-browsecommand) ""]} {
		uplevel #0 $options(-browsecommand) [list [$object get active]]
	}
}

Classy::ListBox method _browse {args} {
	private $object options
	if {"$options(-browsecommand)" != ""} {
		uplevel #0 $options(-browsecommand) [list [$object get active]]
	}
}

Classy::ListBox method insert {args} {
	uplevel #0 $object.list insert $args
}

Classy::ListBox method delete {args} {
	uplevel #0 $object.list delete $args
}

Classy::ListBox method command {} {
	private $object options
	set command [get options(-command) ""]
	if {"$command" != ""} {	
		uplevel #0 $command [list [$object get]]
	}
}

Classy::ListBox method _setvar {} {
	private $object options
	if {![string equal $options(-selvariable) ""]} {
		upvar #0 $options(-selvariable) value
		set temp [$object get]
		if {![info exists value] || ![string equal $value $temp]} {
			set value $temp
		}
	}
}

Classy::ListBox method _traceselvar {name1 name2 op} {
	private $object options
	upvar ::$options(-selvariable) value
	eval {$object set} $value
}

Classy::ListBox method _tracevaluesvar {name1 name2 op} {
	private $object options
	upvar ::$options(-valuesvariable) value
	$object configure -values $value
}

Classy::ListBox method curselection {{values 0}} {
	private $object selection
	if {$values} {
		return [list_sub [$object get 0 end] [$object.list curselection]]
	} else {
		return [$object.list curselection]
	}
}

Classy::ListBox method see {{index {}}} {
	private $object data
	if {[catch {$object index $index} okindex]} {
		set okindex [lsearch -exact [$object get 0 end] $index]
	}
	if {$okindex == -1} {error "element \"$index\" not in listbox"}
	$object.list see $okindex
}

Classy::ListBox method _updown {dir} {
	private $object options
	set docmd 0
	if {$options(-endcommand) ne ""} {
		set pos [expr {[$object.list index active]+$dir}]
		if {$dir > 0} {
			set len [$object.list index end]
			if {$pos >= $len} {
				set docmd 1
			}
		} elseif {$pos < 0} {
			set docmd 1
		}
	}
	if {$docmd} {
		uplevel #0 $options(-endcommand) $dir
	} else {
		tk::ListboxUpDown $object.list $dir
	}
}
